functions {

    real[,] vgpareto_surv(real[] t, real[] mu, real[] alpha, int ROW,int COL) {
       real output[ROW,COL];

       // this is a matrix -> best traversed in row major order
       for (j in 1:ROW){
          for (i in 1:COL){
                output[j,i] =  exp(pareto_type_2_lccdf(t[i] | 0.0, exp(-mu[j]), alpha[j]));
          }
       }

       return output;
    }

   real gpareto_surv(real t, real mu, real alpha) {
        return exp(pareto_type_2_lccdf(t | 0.0, exp(-mu), alpha));
   }

   real log_gpareto_right_censor(real t, real mu, real alpha) {
        return pareto_type_2_lccdf(t | 0.0, exp(-mu), alpha);
   }

   real log_gpareto_left_censor(real t, real mu, real alpha) {
        return pareto_type_2_lcdf(t | 0.0, exp(-mu), alpha);
   }

   real log_gpareto_interval_censor(real tl,real tr, real mu, real alpha) {
        real a[2];
        real c;
        a[1] = pareto_type_2_lccdf(tl | 0.0, exp(-mu), alpha);
        a[2] = pareto_type_2_lccdf(tr | 0.0, exp(-mu), alpha);
        c = max(a);
        return c+ log(exp(a[1]-c) - exp(a[2]-c));
   }

   real log_gpareto_exact_lifetime(real t, real mu, real alpha) {
        return pareto_type_2_lpdf( t |0.0, exp(-mu),  alpha);
   }
}


data {
   int<lower=0> N;
   int<lower=1> N_GROUPS;
   real t[N,2];
   real CENC[N];
   int<lower=1> ID[N];
}


parameters {

   real l;
   real <lower = 0.01> b;
   real <lower=0> lsig_sq;
  // real<lower = 0>  lsig_sq;
   real l_reff[N_GROUPS];

}


//////////////////////////////////////////
model {
     l ~ cauchy(0,1);
     b ~ gamma(2,1);
     lsig_sq ~ inv_gamma(1,1);
     l_reff ~  normal(l,sqrt(lsig_sq) );



     for ( i in 1:N){
           if (CENC[i] == 0)
                 target  +=  log_gpareto_right_censor(t[i,1],l_reff[ID[i]], b);
           if (CENC[i] == 1)
                 target +=   log_gpareto_exact_lifetime(t[i,1],l_reff[ID[i]], b);
           if (CENC[i] == 2)
                 target +=   log_gpareto_left_censor(t[i,1],l_reff[ID[i]], b);
           if (CENC[i] == 3){
                 target  +=  log_gpareto_interval_censor(t[i,1],t[i,2],l_reff[ID[i]], b);
            }
      }
}

generated quantities{
  vector[N] log_lik;

      for(i in 1:N) {
         if (CENC[i] == 0)
               log_lik[i] =  log_gpareto_right_censor(t[i,1],l_reff[ID[i]], b);
         if (CENC[i] == 1)
               log_lik[i] =  log_gpareto_exact_lifetime(t[i,1],l_reff[ID[i]], b);
         if (CENC[i] == 2)
               log_lik[i] =  log_gpareto_left_censor(t[i,1],l_reff[ID[i]],b);
         if (CENC[i] == 3)
               log_lik[i] =  log_gpareto_interval_censor(t[i,1],t[i,2],l_reff[ID[i]], b);

     }

}
