functions { 
   
    real[,] vlnorm_surv(real[] t, real[] mu, real[] sigma, int ROW,int COL) {
       real output[ROW,COL]; 
     
       // this is a matrix -> best traversed in row major order
       for (j in 1:ROW){ 
          for (i in 1:COL){
                output[j,i] =  exp(lognormal_lccdf(t[i] | mu[j], sigma[j])); 
          }
       }  
     
       return output; 
    }
   
   real lnorm_surv(real t, real mu, real sigma) { 
        return exp(lognormal_lccdf(t | mu, sigma)); 
   } 

   real log_lnorm_right_censor(real t, real mu, real sigma) { 
        return lognormal_lccdf(t | mu, sigma); 
   } 
  
   real log_lnorm_left_censor(real t, real mu, real sigma) { 
        return lognormal_lcdf( t | mu, sigma); 
   }

   real log_lnorm_interval_censor(real tl,real tr, real mu, real sigma) { 
        real a[2];
        real c;
        a[1] = lognormal_lccdf(tl | mu, sigma);
        a[2] = lognormal_lccdf(tr | mu, sigma); 
        c = max(a);
        return c+ log(exp(a[1]-c) - exp(a[2]-c)); 
   }

   real log_lnorm_exact_lifetime(real t, real mu, real sigma) { 
        return lognormal_lpdf( t |  mu,  sigma); 
   }  
} 
 

data { 
   int<lower=0> N; 
   int<lower=1> N_GROUPS; 
   real t[N,2];
   real CENC[N];
   int<lower=1> ID[N];
} 
 

parameters {

   real l; 
   real <lower = 0> b; 
   real <lower=0> lsig_sq;
  // real<lower = 0>  lsig_sq;	
   real l_reff[N_GROUPS]; 
   
} 


//////////////////////////////////////////
model { 
     l ~ cauchy(0,1); 
     b ~ gamma(1,1);
     lsig_sq ~ inv_gamma(1,1); 
     l_reff ~  normal(l,sqrt(lsig_sq) ); 
   

     for ( i in 1:N){
        
           if (CENC[i] == 0) 
                 target  +=   log_lnorm_right_censor(t[i,1],l_reff[ID[i]], sqrt(1/b)); 
           if (CENC[i] == 1) 
                 target +=   log_lnorm_exact_lifetime(t[i,1],l_reff[ID[i]], sqrt(1/b)); 
           if (CENC[i] == 2) 
                 target +=   log_lnorm_left_censor(t[i,1],l_reff[ID[i]], sqrt(1/b)); 
           if (CENC[i] == 3){ 
                 target  +=  log_lnorm_interval_censor(t[i,1],t[i,2],l_reff[ID[i]], sqrt(1/b));
            }
      }
      

   
} 

generated quantities{
  vector[N] log_lik; 
  
      for(i in 1:N) { 
         if (CENC[i] == 0) 
               log_lik[i] =    log_lnorm_right_censor(t[i,1],l_reff[ID[i]], sqrt(1/b)); 
         if (CENC[i] == 1) 
               log_lik[i] =   log_lnorm_exact_lifetime(t[i,1],l_reff[ID[i]], sqrt(1/b)); 
         if (CENC[i] == 2) 
               log_lik[i] =   log_lnorm_left_censor(t[i,1],l_reff[ID[i]],sqrt(1/b)); 
         if (CENC[i] == 3) 
               log_lik[i] =  log_lnorm_interval_censor(t[i,1],t[i,2],l_reff[ID[i]], sqrt(1/b));

     } 

}
